# frozen_string_literal: true

module Hermes
  # Threshold clas
  class Threshold
    include Hermes::Concerns::CommonFieldsConcern
    field :type, type: String
    field :thresholds_map, type: Hash

    has_many :housing_offers, class_name: 'Hermes::HousingOffer'

    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'
    owner_field :user_id

    after_create :after_create_actions
    after_update :after_actions

    validates_presence_of :type, :thresholds_map
    #TODO : handle string data for thresholds_map
    def subtype(subtype_name)
      thresholds_map[subtype_name]
    end

    def thresholds_map=(h)
      super(
        if h.is_a? Hash
          h.all_to(Integer)
        else
          {}
        end
      )
    end

    def thresholds(subtype_name, type_name)
      sub_type = subtype(subtype_name)
      return unless sub_type

      field_name = case type_name
                   when 'ceils'
                     'thresholds'
                   else
                     type_name
                   end
      sub_type["#{field_name}_list"]
    end

    def threshold(subtype_name, type_name, index)
      thresholds = thresholds(subtype_name, type_name)
      return unless thresholds

      thresholds[index] || 0.0
    end

    def threshold_addition(subtype_name, type_name, index)
      sub_type = subtype(subtype_name)
      thresholds = thresholds(subtype_name, type_name)
      return unless thresholds

      addition_value = sub_type['another_person']
      return unless addition_value

      thresholds.last + (addition_value * index) || 0.0
    end

    def udpate_housing_offers
      self.housing_offers = Hermes::HousingOffer.where(threshold_type: type)
    end

    def set_offer_thresholds(offer)
      offer_subtype = offer.threshold_subtype
      return unless offer_subtype

      %w[ceils floors].each do |field_type|
        (1..6).each do |i|
          offer.send(
            "#{field_type.singularize}_category_#{i}=",
            threshold(offer_subtype, field_type, i - 1) || 0.0
          )
        end
        (7..12).each do |i|
          j = i - 6
          offer.send(
            "#{field_type.singularize}_category_#{i}=",
            threshold_addition(offer_subtype, field_type, j) || 0.0
          )
        end
      end
    end

    def update_related_offers
      housing_offers.each do |offer|
        set_offer_thresholds(offer)
        offer.save
      end
    end

    def after_create_actions
      udpate_housing_offers
      update_related_offers
    end

    def after_actions
      update_related_offers if thresholds_map_changed?
    end
  end
end
