# frozen_string_literal: true

module Hermes
  module BoClient
    # extending Mapping Module from base_map.rb
    module Mappings
      # handle HousingRequest model mapping for xml data send
      class HousingRequestMap
        include Hermes::BoClient::Mappings::Concerns::BaseMap
        
        def initialize(options)
          housing_request_id = options[:housing_request_id]
          @numUnique = options[:num_unique]
          @main_object = Hermes::HousingRequest.find_by(id: housing_request_id)
          @nud = options[:nud_id] ? Hermes::Nud.find_by(id: options[:nud_id]) : false
          self.main_object = @main_object
          @map = {
              
              "refInterne": "als_id",
              
              
              "montantLoyer": "seeked_max_rent",
              
              
              "codePostalLoc": "seeked_district_postal_codes",
              "nom": "requester_lastname",
              "prenom": "requester_firstname",
              
              "mel": "requester_email",
              
              "anneeMoins1": "main_applicant.year_minus_one",
              
              "listeRessourceRecue": "main_applicant&.monthly_incomes",
              "listeRessourceVersee": "main_applicant&.monthly_charges",
              "listeRessourceRecuePersACharge": "dependents",
              "listeRessourceVerseePersACharge": "dependents",
            
          }
        end
        
        {
            
            "appartementLog": "main_applicant&.actual_housing_details&['appartment_number']",
            "etageLog": "main_applicant&.actual_housing_details&['appartment_floor']",
            "escalierLog": "main_applicant&.actual_housing_details&['appartment_staircase']",
            "batimentLog": "main_applicant&.actual_housing_details&['appartment_building']",
            "compAdresseLog": "main_applicant&.actual_housing_details&['address_add_address']",
            "lieuDitLog": "main_applicant&.actual_housing_details&['appartment_address_place']",
            "numeroLog": "main_applicant&.actual_housing_details&['address_number']",
            
            
            "nomHebergeantLog": "main_applicant&.actual_housing_details&['address_number']",
            
            
            "montantMensuelAPL": "main_applicant&.actual_housing_details&['housing_apl']",
            "nombreHabitant": "main_applicant&.actual_housing_details&['housing_resident_nb']",
            "surface": "main_applicant&.actual_housing_details&['housing_surface']"
        }.each do |key, val|
          define_method key.to_sym do
            s = val.split("&['")
            splitted = []
            s.each do |str|
              splitted << str.split("']")[0]
            end
            if splitted.length == 2
              if splitted[0].include? "main_applicant&.actual_housing_details"
                return @main_object.main_applicant&.actual_housing_details ? @main_object.main_applicant&.actual_housing_details[splitted[1]] : nil
              elsif (@main_object.send(splitted[0]))
                return @main_object.send(splitted[0])[splitted[1]]
              end
            end
            if splitted.length == 3
              if splitted[0].include? "main_applicant&.actual_housing_details"
                return @main_object.main_applicant&.actual_housing_details ? @main_object.main_applicant&.actual_housing_details[splitted[1]][splitted[2]] : nil
              elsif (@main_object.send(splitted[0]) && @main_object.send(splitted[0])[splitted[1]])
                return @main_object.send(splitted[0])[splitted[1]][splitted[2]]
              end
            end
          end
        end
        
        def localisationSouhaite
          list = @main_object.seeked_localisation_list
          res = []
          list&.each do |item|
            res_item = {
                code: "75113",#(item["code"] && item["code"].length == 5) ? item["code"] : "75113",
                codePostal: "75013"#(item["codePostal"] && item["codePostal"].length == 5) ? item["codePostal"] : "75013"
            }
            res.push(res_item)
          end
          res
        end
        
        def appartementDem
          if(@main_object.postal_details && @main_object.postal_details != {})
          @main_object.postal_details&.fetch('appartment_number', nil)
          else
            @main_object.actual_housing_details&.fetch('appartment_number', nil)
            end
        end
        
        def etageDem
          if(@main_object.postal_details && @main_object.postal_details != {})
          @main_object.postal_details&.fetch('appartment_floor', nil)
          else
            @main_object.actual_housing_details&.fetch('appartment_floor', nil)
          end
        end
        
        def escalierDem
          if(@main_object.postal_details && @main_object.postal_details != {})
          @main_object.postal_details&.fetch('appartment_staircase', nil)
          else
            @main_object.actual_housing_details&.fetch('appartment_staircase', nil)
          end
        end
        
        def batimentDem
          if(@main_object.postal_details && @main_object.postal_details != {})
          @main_object.postal_details&.fetch('appartment_building', nil)
          else
            @main_object.actual_housing_details&.fetch('appartment_building', nil)
          end
        end
        
        def compAdresseDem
          if(@main_object.postal_details && @main_object.postal_details != {})
          @main_object.postal_details&.fetch('address_add_address', nil)
          else
            @main_object.actual_housing_details&.fetch('address_add_address', nil)
          end
        end
        
        def lieuDitDem
          if(@main_object.postal_details && @main_object.postal_details != {})
        
          @main_object.postal_details&.fetch('appartment_address_place', nil)
          else
            @main_object.actual_housing_details&.fetch('appartment_address_place', nil)
          end
        end
        
        def numeroDem
          if(@main_object.postal_details && @main_object.postal_details != {})
          @main_object.postal_details&.fetch('address_number', nil)
          else
            @main_object.actual_housing_details&.fetch('address_number', nil)
          end
        end
        
        
        def nomHebergeantDem
          if(@main_object.postal_details && @main_object.postal_details != {})
          @main_object.postal_details&.fetch('address_number', nil)
          else
            @main_object.actual_housing_details&.fetch('address_number', nil)
          end
        end
        
        def voieDem
          if(@main_object.postal_details && @main_object.postal_details != {})
          @main_object.postal_details&.fetch('address_street', nil)
          else
            @main_object.actual_housing_details&.fetch('address_street', nil)||@main_object.actual_housing_details&.fetch('way', nil)
          end
        end
        
        def numGuichet
          serverUrl = APP_CONFIG["app_host"]
          if serverUrl == "PROD" #TODO => passer l'url de prod dès qu'on l'a
            return "ACLOG"
          else
            return "G1175"
          end
        end
        
        def codePostalDem
          if(@main_object.postal_details && @main_object.postal_details != {})
          @main_object.postal_details&.fetch('address_code', nil)
          else
            @main_object.actual_housing_details&.fetch('address_code', nil)
          end
          return "75013"
        end
        
        def communeDem
          if(@main_object.postal_details && @main_object.postal_details != {})
          @main_object.postal_details&.fetch('address_insee_code', nil)
          else
            @main_object.actual_housing_details&.fetch('address_insee_code', nil)
          end
          return "75113"
        end
        
        def codePostalLog
          @main_object.main_applicant&.actual_housing_details&.fetch('address_code', nil)
          return "75013"
        end
        
        def communeLog
          @main_object.main_applicant&.actual_housing_details&.fetch('address_insee_code', nil)
          return "75113"
        end
        
        def voieLog
          @main_object.main_applicant&.actual_housing_details&.fetch('address_street', nil) || @main_object.main_applicant&.actual_housing_details&.fetch('way', nil)
        end
        
        def forcerDoublon
          @main_object.forcing_duplicate ? 1 : 0
          return 0
        end
        
        def modeleLogement
          @main_object.main_applicant&.actual_housing_details&.fetch('housing_typology', 'T1') || 'T1'
        end
        
        
        def montantMensuelLoyer
          @main_object.main_applicant&.actual_housing_details&.fetch('housing_rent_amount', "0") || "0"
        end
        
        def proprietaire
          @main_object.is_house_owner ? 1 : 0;
        end
        
        def demandeElargie
          @main_object.widen_request ? 1 : 0;
        end
        
        def rezDeChaussee
          
          @main_object.seeking_first_floor ? 1 : 0;
        end
        
        def sansAscenseur
          @main_object.seeking_elevator ? 1 : 0;
        end
        
        def multiEmployeur
          @main_object.main_applicant&.has_other_employers ? 1 : 0;
        
        end
        
        def renouvellementElec
          return 1 if @main_object.electronical_renewal === nil
          @main_object.electronical_renewal ? 1 : 0
        end
        
        def precision
          @main_object.precisions ? @main_object.precisions : ""
        end
        
        def dateCreationDemande
          if @nud
            @nud.creation_date&.to_date
          else
            @main_object.created_at.to_date
          end
        
        end
        
        def dateNaissance
          @main_object.main_applicant.birth_date&.to_date
        end
        
        def montantMoins1
          @main_object.main_applicant.amount_year_minus_one&.round.to_s
        end
        
        def etrangerDem
          if @main_object.postal_details && @main_object.postal_details["etranger"]
            return 1
          else
            return 0
          end
        end
        
        def etrangerLog
          if @main_object.main_applicant&.actual_housing_details && @main_object.main_applicant&.actual_housing_details["etranger"]
            return 1
          else
            return 0
          end
        end
        
        def etrangerLogAct
          if @main_object.main_applicant&.actual_housing_details && @main_object.main_applicant&.actual_housing_details["etranger"]
            return 1
          else
            return 0
          end
        end
        
        def typeLogement
          @main_object.seeked_types || ["T1"]
        end
        
        def civilite
          @main_object.main_applicant&.civility
        
        end
        
        def nationalite
          @main_object&.main_applicant&.nationality
          return "FRA"
        end
        
        def situationFamiliale
          @main_object&.main_applicant&.family_situation
        end
        
        def categorieLogementRecherche
          res = @main_object&.seeked_housing_categories&.fetch(0, nil) || "J"
          return "J" if res == ""
          res
        end
        
        def typeContratTravail
          @main_object&.main_applicant&.work_contract_type
          return "RET"
        end
        
        def motifDemande
          motifs = []
          motifs.push(@main_object&.motivation1) if @main_object&.motivation1
          motifs.push(@main_object&.motivation2) if @main_object&.motivation2
          motifs.push(@main_object&.motivation3) if @main_object&.motivation3
          motifs
        end
        
        def numUnique
          if (@numUnique)
            numUnique
          elsif @nud
            @nud.unique_number
          end
        end
      end
    end
  end
end
